@@ -13,7 +13,7 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
             super=NULL, spamPivot="MMD", in_coef=0.1, type="MC",
             correct=TRUE, trunc=TRUE, SE_method="LU", nrho=200,
             interpn=2000, small_asy=TRUE, small=1500, SElndet=NULL,
-            LU_order=FALSE, pre_eig=NULL)
+            LU_order=FALSE, pre_eig=NULL, glht=FALSE)
         nmsC <- names(con)
         con[(namc <- names(control))] <- control
         if (length(noNms <- namc[!namc %in% nmsC])) 
@@ -29,6 +29,7 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
 #        stopifnot(is.logical(con$super))
         stopifnot(is.logical(con$compiled_sse))
         stopifnot(is.character(con$spamPivot))
+        stopifnot(is.logical(con$glht))
         if (!inherits(formula, "formula")) formula <- as.formula(formula)
 #	mt <- terms(formula, data = data)
 #	mf <- lm(formula, data, na.action=na.action, method="model.frame")
@@ -40,6 +41,12 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
         mf[[1]] <- as.name("model.frame")
         mf <- eval(mf, parent.frame())
         mt <- attr(mf, "terms")
+        if (attr(mt, "intercept") == 1 && !any(attr(mt, "factors") == 1) &&
+            (is.formula(Durbin) || isTRUE(Durbin))) {
+            warning("intercept-only model, Durbin invalid and set FALSE")
+            Durbin <- FALSE
+        }
+
 #
 	na.act <- attr(mf, "na.action")
 	if (!is.null(na.act)) {
@@ -53,10 +60,11 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
         if (missing(etype))etype <- "error"
         if (etype == "Durbin") etype <- "emixed"
         if (missing(Durbin)) Durbin <- ifelse(etype == "error", FALSE, TRUE)
-        if (listw$style != "W" && is.formula(Durbin)) {
-            Durbin <- TRUE
-            warning("formula Durbin requires row-standardised weights; set TRUE")
-        }
+# FIXME does this hold?
+#        if (listw$style != "W" && is.formula(Durbin)) {
+#            Durbin <- TRUE
+#            warning("formula Durbin requires row-standardised weights; set TRUE")
+#        }
         if (is.logical(Durbin) && isTRUE(Durbin)) etype <- "emixed"
         if (is.formula(Durbin)) etype <- "emixed"
         if (is.logical(Durbin) && !isTRUE(Durbin)) etype <- "error"
@@ -95,6 +103,8 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
             fdHess <- NULL
         }
         stopifnot(is.logical(con$fdHess))
+        dvars <- c(NCOL(x), 0L)
+
 	if (is.formula(Durbin) || isTRUE(Durbin)) {
                 prefix <- "lag"
                 if (isTRUE(Durbin)) {
@@ -165,7 +175,8 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
         sw <- sqrt(weights)
 #
         LL_null_lm <- NULL
-	if ("(Intercept)" %in% colnames(x)) LL_null_lm <- logLik(lm(y ~ 1))
+	if ("(Intercept)" %in% colnames(x)) LL_null_lm <- logLik(lm(y ~ 1,
+            weights=weights))
 	m <- NCOL(x)
 	xcolnames <- colnames(x)
 	K <- ifelse(xcolnames[1] == "(Intercept)", 2, 1)
@@ -261,27 +272,25 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
         emixedImps <- NULL
 	if (etype == "emixed") {
           if (isTRUE(Durbin)) {
-            odd <- (m%/%2) > 0
-            if (odd) {
+            m.1 <- m > 1
+            if (m.1 && K == 2) {
                 m2 <- (m-1)/2
             } else {
                 m2 <- m/2
             }
-            if (K == 1 && odd) {
-                warning("model configuration issue: no total impacts")
-            } else {
                 cm <- matrix(0, ncol=m, nrow=m2)
                 if (K == 2) {
-                    if (odd) {
+                    if (m.1) {
                         rownames(cm) <- xxcolnames[2:(m2+1)]
                     } else {
                         rownames(cm) <- xxcolnames[1:m2]
                     }
-                    for (i in 1:m2) cm[i, c(i+1, i+(m2+1))] <- 1
+                    LI <- ifelse(!(listw$style == "W"), 1, 0) #TR: lagged intercept
+                    for (i in 1:m2) cm[i, c(i+1, i+(m2+1 + LI)) ] <- 1 #TR: Add to index
 # drop bug fix 2016-09-21 Philipp Hunziker
                     dirImps <- sum_lm_target$coefficients[2:(m2+1), 1:2, drop=FALSE]
                     rownames(dirImps) <- rownames(cm)
-                    indirImps <- sum_lm_target$coefficients[(m2+2):m, 1:2, drop=FALSE]
+                    indirImps <- sum_lm_target$coefficients[((m2 + 2):m + LI), 1:2, drop=FALSE] #TR: Add to index
                     rownames(indirImps) <- rownames(cm)
                 } else {
                     rownames(cm) <- xxcolnames[1:m2]
@@ -291,29 +300,36 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
                     indirImps <- sum_lm_target$coefficients[(m2+1):m, 1:2, drop=FALSE]
                     rownames(indirImps) <- rownames(cm)
                 }
-            }
-            totImps <- as.matrix(estimable(lm.target, cm)[, 1:2, drop=FALSE])
+            lc <- summary(multcomp::glht(lm.target, linfct=cm))
+            totImps <- cbind("Estimate"=lc$test$coefficients, "Std. Error"=lc$test$sigma)
           } else if (is.formula(Durbin)) {
 #FIXME
+            LI <- ifelse(listw$style != "W" 
+                         && attr(terms(Durbin), "intercept") == 1, 1, 0)
               m <- sum(dvars)
-              m2 <- dvars[2]
+              KIL <- max((LI - (K - 1)), 0)
+              m2 <- dvars[2] - KIL
               cm <- matrix(0, ncol=m, nrow=m2)
               for (i in 1:m2) {
-                  cm[i, c(inds[i], i+dvars[1])] <- 1
+                  cm[i, c(inds[i], i+dvars[1] + KIL)] <- 1
               }
-              rownames(cm) <- wxn
-              dirImps <- sum_lm_target$coefficients[2:dvars[1], 1:2,
+              if (LI == 1 && K == 1) { 
+                rownames(cm) <- wxn[!grepl("Intercept", wxn)]
+              } else {
+                rownames(cm) <- wxn
+              }
+              dirImps <- sum_lm_target$coefficients[K:dvars[1], 1:2,
                 drop=FALSE]
               rownames(dirImps) <- xn
-              indirImps <- sum_lm_target$coefficients[(dvars[1]+1):m, 1:2,
+              indirImps <- sum_lm_target$coefficients[(dvars[1] + 1 + KIL):m, 1:2,
                 drop=FALSE]
               if (!is.null(zero_fill)) {
                 if (length(zero_fill) > 0L) {
                  lres <- vector(mode="list", length=2L)
                  for (j in 1:2) {
-                  jindirImps <- rep(as.numeric(NA), (dvars[1]-1))
+                  jindirImps <- rep(as.numeric(NA), (dvars[1] + (1-K)))
                   for (i in seq(along=inds)) {
-                    jindirImps[(inds[i]-1)] <- indirImps[i, j]
+                    jindirImps[(inds[i]+(1-K))] <- indirImps[i, j]
                   }
                   lres[[j]] <- jindirImps
                  }
@@ -321,15 +337,15 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
                 }
               }
               rownames(indirImps) <- xn
-              totImps <- as.matrix(estimable(lm.target, cm)[, 1:2,
-                drop=FALSE])
+              lc <- summary(multcomp::glht(lm.target, linfct=cm))
+              totImps <- cbind("Estimate"=lc$test$coefficients, "Std. Error"=lc$test$sigma)
               if (!is.null(zero_fill)) {
                 if (length(zero_fill) > 0L) {
                  lres <- vector(mode="list", length=2L)
                  for (j in 1:2) {
                   jtotImps <- dirImps[, j]
                   for (i in seq(along=inds)) {
-                    jtotImps[(inds[i]-1)] <- totImps[i, j]
+                    jtotImps[(inds[i]+(1-K))] <- totImps[i, j]
                   }
                   lres[[j]] <- jtotImps
                  }
@@ -482,8 +498,8 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
                 timings=do.call("rbind", timings)[, c(1, 3)], 
                 f_calls=get("f_calls", envir=env),
                 hf_calls=get("hf_calls", envir=env), intern_classic=iC,
-                pWinternal=pWinternal, weights=weights, emixedImps=emixedImps),
-                class=c("sarlm"))
+                pWinternal=pWinternal, weights=weights, emixedImps=emixedImps,
+                dvars=dvars), class=c("Sarlm"))
         rm(env)
         GC <- gc()
 	if (zero.policy) {
@@ -494,6 +510,7 @@ errorsarlm <- function(formula, data = list(), listw, na.action, weights=NULL,
 	}
 	if (!is.null(na.act))
 		ret$na.action <- na.act
+        if (is.formula(Durbin)) attr(ret, "Durbin") <- deparse(Durbin)
 	ret
 }
 
@@ -855,7 +872,7 @@ lagsarlm <- function(formula, data = list(), listw,
                 timings=do.call("rbind", timings)[, c(1, 3)], 
                 f_calls=get("f_calls", envir=env),
                 hf_calls=get("hf_calls", envir=env), intern_classic=iC),
-                class=c("sarlm"))
+                class=c("Sarlm"))
         rm(env)
         GC <- gc()
 	if (zero.policy) {
@@ -866,6 +883,7 @@ lagsarlm <- function(formula, data = list(), listw,
 	}
 	if (!is.null(na.act))
 		ret$na.action <- na.act
+        if (is.formula(Durbin)) attr(ret, "Durbin") <- deparse(Durbin)
 	ret
 }
 
@@ -1291,7 +1309,7 @@ sacsarlm <- function(formula, data = list(), listw, listw2=NULL, na.action,
             fdHess=fdHess, resvar=asyvar1, listw_style=listw$style,
             optimHess=FALSE, insert=FALSE, interval1=interval1,
             interval2=interval2, timings=do.call("rbind", timings)[, c(1, 3)]),
-            class=c("sarlm"))
+            class=c("Sarlm"))
         rm(env)
         GC <- gc()
         if (is.null(llprof)) ret$llprof <- llprof
@@ -1299,6 +1317,7 @@ sacsarlm <- function(formula, data = list(), listw, listw2=NULL, na.action,
             ret$llprof <- list(grd=llprof, ll=ll_prof, xseq=llrho,
                 yseq=lllambda)
         }
+        if (is.formula(Durbin)) attr(ret, "Durbin") <- deparse(Durbin)
 	if (!is.null(na.act))
 		ret$na.action <- na.act
 	ret
